## Grundlagen

*Dieser allgemeinen Beschreibung liegt ein konkretes Beispiel zugrunde. Die ODD-Datei lässt sich leicht an abweichende Anforderungen anpassen.*

Zur Erstellung des Schemas wird die ODD-Methode genutzt, vgl. dazu [https://tei-c.org/guidelines/customization/getting-started-with-p5-odds/](https://tei-c.org/guidelines/customization/getting-started-with-p5-odds/). 

Als Startpunkt habe ich in [Roma](https://roma2.tei-c.org/) ein `Build up`-Schema angelegt, d.h. ein sehr reduziertes Modell zugrundegelegt, das dann nach Bedarf ausgebaut werden kann.

Ausgehend von den Beispieldateien habe ich in die Datei `my-custom-TEI-schema.odd` die folgenden Module zusätzlich integriert:

```xml
<!-- the analysis module is needed for pc and similar elements -->
<moduleRef key="analysis"/>
<!-- the textcrit module is needed for listWit and similar elements -->
<moduleRef key="textcrit" except=""/>
<!-- the msdescription module is needed for msDesc and similar elements -->
<moduleRef key="msdescription" except=""/>
<!-- the namesdates module is needed for placeName and similar elements -->
<moduleRef key="namesdates" except=""/>
<!-- the transcr module is needed for facs and similar attributes -->
<moduleRef key="transcr" except=""/>
<!-- the linking module is needed for corresp and similar attributes -->
<moduleRef key="linking"/>
```

Zusätzlich werden zwei projektspezifische Attribute definiert:

```xml
<!-- Here we define custom attributes in the project namespace. 
     In this case the pc-Element is extended by two custom attributes. 
     Use @ident="att.global" and @module="tei" to allow the attributes everywhere. -->
<classSpec ident="pc" type="atts" mode="change" module="analysis">
    <attList>
        <attDef ident="n1-subst" ns="my-custom-TEI-schema-ns">
            <desc>n1-subst ...{add your description here; it will be presented as a tooltip in oXygen}...</desc>
            <datatype>
                <rng:text/>
            </datatype>
        </attDef>
        <attDef ident="n2-subst" ns="my-custom-TEI-schema-ns">
            <desc>n2-subst ...{add your description here; it will be presented as a tooltip in oXygen}...</desc>
            <datatype>
                <rng:text/>
            </datatype>
        </attDef>
    </attList>
</classSpec>
```

oXygen stellt diese Attribute über die Auto-complete-Funktionalität zur Verfügung (basierend auf `@ident`).

![image](autocomplete.png)

## Anwendung

### Vorbereitung:

- In den mit dem Schema zu validierenden XML-Dateien ist die Modell-Deklaration 
    ```xml
    <?xml-model href="http://www.tei-c.org/release/xml/tei/custom/schema/relaxng/tei_all.rng" type="application/xml" schematypens="http://relaxng.org/ns/structure/1.0"?>
    ``` 
    umzuändern in 
    ```xml
    <?xml-model href="my-custom-TEI-schema.rng" type="application/xml" schematypens="http://relaxng.org/ns/structure/1.0"?>
    ```

- Die XML-Dateien müssen zudem eine entsprechende Namespace-Deklaration erhalten (z.B. `xmlns:my="my-custom-TEI-schema-ns"`).

### Bearbeitung und Generierung des Schemas:

- Datei `my-custom-TEI-schema.odd` in oXygen öffnen.
- Bearbeitungen vornehmen (Module ergänzen, eigene Attribute ergänzen, usw.).
- Das oXygen-Transformationsszenario `TEI ODD to RELAX NG XML` laufen lassen (mit der ODD-Datei als Input).
![odd-ant](odd-ant.png)
- Es resultiert eine Datei `my-custom-TEI-schema.rng`; diese muss ins Projektverzeichnis verschoben werden (ggf. existierende Datei überschreiben).

### Beispiel-Dateistuktur

```
/Users/examples/elisa
├── cgm696_test_punteggiatura-3.xml
├── inc77_test_punteggiatura.xml
├── out 					## Verzeichnis, in das das generierte Schema gespeichert wird
├── my-custom-TEI-schema.odd 			## Ausgangsdatei für das Schema
└── my-custom-TEI-schema.rng			## Generiertes Schema
```

